
/**************************************************************

   HttpClient class.

    This class provides support for GET and POST requests with
    failover and redirects..

**************************************************************/

var debug = false;
//stdout = WScript.stdout;

//-- HTTP states ---
var HTTP_STATE_NONE  = 0;
var HTTP_STATE_PRIMARY = 1;
var HTTP_STATE_REDIRECT = 2;
var HTTP_STATE_FAILOVER = 3;

var HTTP_VERB_GET = 'GET';
var HTTP_VERB_POST = 'POST';

function HttpClient( )
{
}
HttpClient.prototype =
{
    verb: 'NONE',
    url : '',
    failoverUrl: '',
    contentType: '',
    xmlhttp:false,
    requestData: '',
    state: HTTP_STATE_NONE,
    onSuccess: false,
    onError: false,
    onRedirect: false,
    onFailover: false,

    init:function( )
    {
        state = HTTP_STATE_NONE;
        try
        {
            // Mozilla / Safari / IE7
            this.xmlhttp = new XMLHttpRequest( );
        }
        catch( e )
        {
            var XMLHTTP_IDS = new Array('MSXML2.XMLHTTP.5.0',
                                        'MSXML2.XMLHTTP.4.0',
                                        'MSXML2.XMLHTTP.3.0',
                                        'MSXML2.XMLHTTP',
                                        'Microsoft.XMLHTTP' );
            var success = false;
            for( var i = 0; i < XMLHTTP_IDS.length && !success; i++ )
            {
                try
                {
                     this.xmlhttp = new ActiveXObject( XMLHTTP_IDS[i] );
                     success = true;
                }
                catch( e )
                {
                }
             }
             if ( !success )
             {
                 throw new Error('Unable to create XMLHttpRequest.');
             }
         }
    },

    sendRequest:function( verb, url, failoverUrl, contentType, requestData )
    {
        this.verb = verb;
        this.url = url;
        this.failoverUrl = failoverUrl;
        this.contentType = contentType;
        this.requestData = requestData;

        if ( !this.xmlhttp )
        {
            this.init( );
        }
        try
        {
            if ( this.state == HTTP_STATE_NONE )
            {
               this.state = HTTP_STATE_PRIMARY;
            }
            // set onreadystatechange here since it will be reset after a
            //completed call in Mozilla
            var self = this;
            this.xmlhttp.onreadystatechange = function( )
            {
                self.processResponse( );
            }

            var requestURL = this.url;
            if ( verb == HTTP_VERB_GET && requestData != '' )
            {
                requestURL = requestURL + '?' + requestData;
            }

            // send async request
            this.xmlhttp.open( this.verb, requestURL, true );
            if ( verb == HTTP_VERB_GET )
            {
                this.xmlhttp.send( null );
            }
            else
            {
                this.xmlhttp.send( requestData );
            }
        }
        catch( e )
        {
            if ( this.state == HTTP_STATE_PRIMARY )
            {
                this.failover( );
                return;
            }
            this.onError( 'Error :' + e + ' sending request: ' + this.url );
            this.reset( );
        }
    },

    processResponse:function( )
    {
        if ( this.xmlhttp.readyState == 4 ) // only if req shows 'loaded'
        {
            if ( this.xmlhttp.status == 200 )
            {
                if ( this.xmlhttp.responseXML == null )
                {
                    this.onError( "XML response is empty. Response = " + this.xmlhttp.responseText );
                }
                else
                {
                    this.onSuccess( this.xmlhttp.responseXML );
                }

                this.reset( );
            }
            else if( this.xmlhttp.status == 301 || this.xmlhttp.status == 302 || this.xmlhttp.status == 305 )
            {
                // handle redirect
                this.redirect( );
            }
            else
            {
                if ( this.state == HTTP_STATE_PRIMARY )
                {
                    this.failover( );
                    return;
                }
                this.reset( );
                this.onError( 'Unable to process HTTP request. Please try again later.' );
            }
        }
    },

    failover:function( )
    {
        this.state = HTTP_STATE_FAILOVER;
        if ( this.onFailover != false )
        {
            this.onFailover( this.failoverUrl );
        }
        this.sendRequest( this.verb, this.failoverUrl, this.url, this.contentType, this.requestData );
    },

    redirect:function( )
    {
        try
        {
            var newLocation = this.xmlhttp.getResponseHeader( 'Location' );
            this.state = HTTP_STATE_REDIRECT;
            this.url = newLocation;
            if ( this.onRedirect != false )
            {
                this.onRedirect( newLocation );
            }
            this.sendRequest( this.verb, this.url, this.failoverUrl, this.contentType, this.requestData );
        }
        catch( e )
        {
            this.onError( 'Unable to handle redirect:' + e );
            this.reset( );
        }
    },

    reset:function( )
    {
        this.verb = '';
        this.url = '';
        this.contentType = '';
        this.requestData = '';
        this.state = HTTP_STATE_NONE;
        this.xmlhttp = false;
    }
}
