/******************************************************************************
    CallWave SMS Widget

    Send free SMS text messages worldwide.

    Copyright (C) 2006 CallWave, Inc.  All Rights Reserved.

******************************************************************************/

/******************************************************************************
  Constants
******************************************************************************/
var CONTACT_NONE_SELECTED_VALUE         = "---none---";
var WIDGET_VERSION                      = "3.4";
var WIDGET_SOURCE                       = "widget";

var CLIENT_VERSION_CHECK_URL            = "http://www.callwave.com/sms/checkForUpdates.html";
var HELP_URL                            = "http://www.callwave.com/sms/mainfaq.asp";

// preference keys
var LAST_CONTACT_KEY                    = "Last Contact";
var MODE_KEY                            = "Mode";
var NUM_SENT_KEY                        = "Num Sent";
var SENDER_CARRIER_ID_KEY               = "Sender Carrier Id";
var SENDER_EMAIL_KEY                    = "Sender Email";
var SENDER_MOBILE_KEY                   = "Sender Mobile";
var SENDER_FROM_PREFERENCE_KEY          = "Sender From Preference";

var SEND_FROM_USE_PHONE_NUMBER          = 0;
var SEND_FROM_USE_EMAIL_ADDRESS         = 1;

var MODE_DOMESTIC                       = "free";
var MODE_INTERNATIONAL                  = "worldwide";

var OK_RESPONSE                         = 'OK';

var HTTP_PRIMARY_URL                    = "http://sms0.callwave.com";
var HTTP_FAILOVER_URL                   = "http://sms0.gotpb.com";

var HTTP_CONTEXT_ROOT_DOMESTIC          = "smscw";
var HTTP_CONTEXT_ROOT_INTERNATIONAL     = "smscwi";

var HTTP_CONTENT_TYPE_APPLICATION_FORM  = "application/x-www-form-urlencoded";

/******************************************************************************
  Globals
******************************************************************************/
var messageDimensions                   = null;
var lastSentMessageMessagePartCount     = 0;
var totalDomesticMessagePartsSent       = 0;

function MessageDimensions( )
{
    // private constants
    var MAX_MESSAGE_PARTS  = 5;
    var MAX_MESSAGE_LENGTH = 158;
    var CARRIER_OVERHEAD   = 2;

    this.MAX_INTERNATIONAL_MESSAGE_LENGTH        = 160;
    this.MAX_UNICODE_MESSAGE_LENGTH              = 70;

    var senderMobile = widget.preferenceForKey( SENDER_MOBILE_KEY );
    var senderEmail  = widget.preferenceForKey( SENDER_EMAIL_KEY );

    var replyAddressLength = 38;
    if ( senderMobile && ( senderMobile.length > 0 ) )
    {
        replyAddressLength = senderMobile.length + 23;
    }
    else if ( senderEmail && ( senderEmail.length > 0 ) )
    {
        replyAddressLength = senderEmail.length;
    }

    var singlePartOverheadLength     = replyAddressLength + CARRIER_OVERHEAD;
    this.maxSinglePartLength         = MAX_MESSAGE_LENGTH - singlePartOverheadLength;

    var multiPartOverheadLength      = singlePartOverheadLength + "x of x: ".length;
    this.maxMultiPartLength          = MAX_MESSAGE_LENGTH - multiPartOverheadLength;
    this.maxTotalMessageLength       = MAX_MESSAGE_PARTS * this.maxMultiPartLength;

    this.MAX_INTERNATIONAL_MESSAGE_PART_LENGTH   = this.MAX_INTERNATIONAL_MESSAGE_LENGTH - 8;
    this.MAX_UNICODE_MESSAGE_PART_LENGTH         = this.MAX_UNICODE_MESSAGE_LENGTH - 8;
    this.MAX_TOTAL_INTERNATIONAL_MESSAGE_LENGTH  = this.MAX_MESSAGE_PARTS * this.MAX_INTERNATIONAL_MESSAGE_PART_LENGTH;
    this.MAX_TOTAL_UNICODE_MESSAGE_LENGTH        = this.MAX_MESSAGE_PARTS * this.MAX_UNICODE_MESSAGE_PART_LENGTH;

    this.LimitDomesticMessageLength = function ( messageObj )
    {
        var message = messageObj.value;

        var messageLength = message.length;

        if ( messageLength <= this.maxTotalMessageLength )
        {
            return messageLength;
        }

        messageObj.value = message.substr( 0, this.maxTotalMessageLength );

        return this.maxTotalMessageLength;
    }

    this.DetermineDomesticMessagePartCount = function ( messageLength )
    {
        if ( messageLength > this.maxSinglePartLength )
        {
            return Math.ceil( messageLength / this.maxMultiPartLength );
        }

        return 1;
    }

    this.LimitInternationalMessageLength = function ( messageObj )
    {
        if ( messageObj.value.length > this.MAX_TOTAL_INTERNATIONAL_MESSAGE_LENGTH )
        {
            messageObj.value = messageObj.value.substr( 0, this.MAX_TOTAL_INTERNATIONAL_MESSAGE_LENGTH );
        }
    };

    this.DetermineIntlMessageUnicodePartCount = function( messageLength )
    {
        if ( messageLength > this.MAX_TOTAL_UNICODE_MESSAGE_LENGTH )
        {
            SetStatusMessage( '<font color="red">Unicode messages can only have ' + this.MAX_TOTAL_UNICODE_MESSAGE_LENGTH + ' characters</font>' );
        }

        if ( messageLength > this.MAX_UNICODE_MESSAGE_LENGTH )
        {
            return Math.ceil( messageLength / this.MAX_UNICODE_MESSAGE_PART_LENGTH );
        }

        return 1;
    };

    this.DetermineIntlMessagePartCount = function( messageLength )
    {
        if ( messageLength > this.MAX_INTERNATIONAL_MESSAGE_LENGTH )
        {
            return Math.ceil( messageLength / this.MAX_INTERNATIONAL_MESSAGE_PART_LENGTH );
        }

        return 1;
    };
}

/******************************************************************************
 ******************************************************************************

  Initialization functions

 ******************************************************************************
******************************************************************************/
function Setup( )
{
    MigrateSettings( );

    // populate the popups
    FillContactsPopup( );
    PopulateCarrierNames( document.getElementById( 'carrierPopup' ),     GetRecipientCarrierList( ) );
    PopulateCarrierNames( document.getElementById( 'senderCarrierPopup' ), GetSenderCarrierList( ) );

    totalDomesticMessagePartsSent = widget.preferenceForKey( NUM_SENT_KEY );
    if ( ! totalDomesticMessagePartsSent || ( ( "" + totalDomesticMessagePartsSent ).length == 0 ) )
    {
        totalDomesticMessagePartsSent = 0;
    }

    InitializeBackPane( );
    TellUserToEnterReplyAddress( );

    messageDimensions = new MessageDimensions( );

    var mode = widget.preferenceForKey( MODE_KEY );

    var modeId = '';

    if ( mode == MODE_INTERNATIONAL )
    {
        document.getElementById( 'internationalRadio' ).checked = true;
        document.getElementById( 'domesticRadio' ).checked = false;

        UpdateInternationalCounts( );

        modeId = 'internationalRadio';
    }
    else
    {
        document.getElementById( 'domesticRadio' ).checked = true;
        document.getElementById( 'internationalRadio' ).checked = false;

        UpdateDomesticCounts( );

        modeId = 'domesticRadio';
    }

    widget.onshow = OnShow;

    var lastContact = widget.preferenceForKey( LAST_CONTACT_KEY );
    if ( lastContact && lastContact.length > 0 )
    {
        var contactPopup = document.getElementById( "contactPopup" );
        SelectItemInCustomPopup( lastContact, contactPopup, document.getElementById( "contactMenuText" ) );

        OnContactChanged( );
    }

    OnModeChanged( modeId );
}

function MigrateSettings( )
{
    var savedSenderCarrier = widget.preferenceForKey( "Sender Carrier" );
    if ( ! savedSenderCarrier || savedSenderCarrier.length == 0 )
    {
        return;
    }

    var carrierId = FindCarrierId( savedSenderCarrier );
    if ( carrierId != INVALID_CARRIER_ID )
    {
        widget.setPreferenceForKey( carrierId, SENDER_CARRIER_ID_KEY );
    }

    // clear the old deprecated preference
    widget.setPreferenceForKey( '',  "Sender Carrier" );
}

function UpdateCounts( )
{
    if ( document.getElementById( 'internationalRadio' ).checked )
    {
        UpdateInternationalCounts( );
    }
    else
    {
        UpdateDomesticCounts( );
    }
}

// If the user hasn't already entered their mobile info or email address on the back
// of the widget, this sets an error message encouraging them to do so.
function TellUserToEnterReplyAddress( )
{
    if ( ! SenderInformationExists( ) )
    {
        SetStatusMessage( "Click the 'i' and enter a reply address!" );
    }
}

function InitializeBackPane( )
{
    // create buttons
    var reloadButton = document.getElementById( 'reloadButton' );
    createGenericButton( reloadButton, 'Reload Contacts', OnReloadContacts, 67 );

    var versionCheckButton = document.getElementById( 'versionCheckButton' );
    createGenericButton( versionCheckButton, 'Check for updates', OnCheckForUpdates, 87 );

    var helpButton = document.getElementById( 'helpButton' );
    createGenericButton( helpButton, 'Help', OnHelp, 50 );

    var doneButton = document.getElementById( 'doneButton' );
    createGenericButton( doneButton, 'Done', OnBackPaneDoneButton, 50 );

    var senderSavedEmail = widget.preferenceForKey( SENDER_EMAIL_KEY );
    if ( senderSavedEmail )
    {
        document.getElementById( 'senderEmail' ).value = senderSavedEmail;
    }

    var senderPhoneNumber = widget.preferenceForKey( SENDER_MOBILE_KEY );
    if ( senderPhoneNumber && ( senderPhoneNumber.length >= 10 ) )
    {
        document.getElementById( "senderPhoneNumber" ).value = senderPhoneNumber;
    }

    var savedSenderCarrierId = widget.preferenceForKey( SENDER_CARRIER_ID_KEY );
    if ( savedSenderCarrierId )
    {
        SelectItemInPopup( savedSenderCarrierId, document.getElementById( 'senderCarrierPopup' ) );
    }

    var savedSendFromPreference = widget.preferenceForKey( SENDER_FROM_PREFERENCE_KEY );
    if ( savedSendFromPreference && ( savedSendFromPreference == SEND_FROM_USE_EMAIL_ADDRESS ) )
    {
        document.getElementById( 'sendFromPreferenceCheckBox' ).checked = true;
    }
}


/******************************************************************************
  Event Handlers
******************************************************************************/
function OnShow( )
{
    TellUserToEnterReplyAddress( );

    document.getElementById( 'messageText' ).focus( );
    document.getElementById( 'messageText' ).select( );
}

/******************************************************************************
 ******************************************************************************

  FRONT functionality functions

 ******************************************************************************
******************************************************************************/

/******************************************************************************
  SetStatusMessage

    Sets the status message

******************************************************************************/
function SetStatusMessage( message )
{
    document.getElementById( 'errorMessage' ).innerHTML = message;
}

/******************************************************************************
  OnModeChanged

    Keaka note: there is a bug in 10.4.3 where the other radio buttons aren't
                unchecked when a new one is checked.  This was fixed in 10.4.4, but
                we have to manually uncheck the other button here.

******************************************************************************/
function OnModeChanged( elementId )
{
    SetStatusMessage( '' );

    if ( elementId == "internationalRadio" )
    {
        document.getElementById( 'domesticRadio' ).checked = false;
        document.getElementById( 'carrierDiv' ).style.display = 'none';
        widget.setPreferenceForKey( MODE_INTERNATIONAL, MODE_KEY );

        UpdateInternationalCounts( );
    }
    else
    {
        document.getElementById( 'internationalRadio' ).checked = false;
        document.getElementById( 'carrierDiv' ).style.display = 'block';
        widget.setPreferenceForKey( MODE_DOMESTIC, MODE_KEY );

        UpdateDomesticCounts( );
    }
}

/******************************************************************************
  buttonUpOut

    Keaka note: When the action button is clicked upon or clicked on, moused out,
                and moused up, it needs to be replaced by the unactive version
                of the button.

******************************************************************************/
function buttonUpOut( )
{
    document.getElementById( "doButton" ).src = "Images/Button.png";
}

/******************************************************************************
  OnRecipientCarrierChanged

    Handler for recipient carrier selection-change events

******************************************************************************/
function OnRecipientCarrierChanged( elem )
{
    var chosenCarrierId = elem.options[ elem.selectedIndex ].value;

    if ( chosenCarrierId != INVALID_CARRIER_ID )
    {
        var chosenCarrier = elem.options[ elem.selectedIndex ].text;

        document.getElementById( "carrierMenuText" ).innerText = chosenCarrier;
    }
    else
    {
        SelectFirstItemInCustomPopup( elem, document.getElementById( "carrierMenuText" ) );
    }

    document.getElementById( 'messageText' ).focus( );
    document.getElementById( 'messageText' ).select( );
}

/******************************************************************************
  OnContactChanged

    Handler for contact change events

******************************************************************************/
function OnContactChanged( )
{
    var contactPopup = document.getElementById( "contactPopup" );

    var chosenContact = contactPopup.options[ contactPopup.selectedIndex ].text;

    if ( chosenContact.length <= 17 )
    {
        document.getElementById( "contactMenuText" ).innerText = chosenContact;
    }
    else
    {
        var trimmedContact = chosenContact.substring( 0, 15 ) + "...";
        document.getElementById( "contactMenuText" ).innerText = trimmedContact;
    }

    var chosenContactNumber = contactPopup.options[ contactPopup.selectedIndex ].value;
    if ( chosenContactNumber == CONTACT_NONE_SELECTED_VALUE )
    {
        // We want the number fields to be blank if they select "None" from the contact menu.
        chosenContactNumber = "";
    }
    else
    {
        document.getElementById( 'messageText' ).focus( );
        document.getElementById( 'messageText' ).select( );
    }

    var numberLength = chosenContactNumber.length;
    document.getElementById( "recipientPhoneNumber" ).value = chosenContactNumber;
    SelectCarrierForNumber( );
}

/******************************************************************************
  OnRecipientNumberChanged

    Handler for recipient number change events

******************************************************************************/
function OnRecipientNumberChanged( )
{
    SelectFirstItemInCustomPopup( document.getElementById( "contactPopup" ), document.getElementById( "contactMenuText" ) );

    SelectCarrierForNumber( );
}

/******************************************************************************
  OnKeyDownInMessageBody

    Keaka note: Event handler for onKeyDown for the message field.  Clear any
                error message. In particular, this gets rid of the "Message Sent!"
                message once the user starts typing a new message.

******************************************************************************/
function OnKeyDownInMessageBody( )
{
    SetStatusMessage( "" );
}

/******************************************************************************
  SelectCarrierForNumber

    Selects the carrier of the recipient phone number

******************************************************************************/
function SelectCarrierForNumber( )
{
    if ( ! window.widget )
    {
        return;
    }

    var recipientPhoneNumber = document.getElementById( 'recipientPhoneNumber' ).value;

    if ( ! recipientPhoneNumber || recipientPhoneNumber.length == 0 )
    {
        return;
    }

    var savedCarrier = widget.preferenceForKey( recipientPhoneNumber );

    if ( savedCarrier && savedCarrier.length > 0 )
    {
        var savedCarrierId = INVALID_CARRIER_ID;

        if ( savedCarrier.match( /^\d+$/  ) )
        {
            savedCarrierId = parseInt( savedCarrier );
        }
        else
        {
            savedCarrierId = FindCarrierId( savedCarrier );
        }

        SelectItemInCustomPopup( savedCarrierId, document.getElementById( 'carrierPopup' ), document.getElementById( 'carrierMenuText' ) );
        return;
    }

    SelectFirstItemInCustomPopup( document.getElementById( 'carrierPopup' ),
                                  document.getElementById( "carrierMenuText" ) );
}

/******************************************************************************
  UpdateInternationalCounts

    Updates the international message counts

******************************************************************************/
function UpdateInternationalCounts( )
{
    var message = document.getElementById( 'messageText' );

    messageDimensions.LimitInternationalMessageLength( message );

    document.getElementById( 'characterCount' ).innerHTML = "Chars: " + message.value.length;

    if ( StringRequiresUnicodeEncoding( message.value ) )
    {
        document.getElementById( 'unicodeIndicator' ).style.display = 'block';

        lastSentMessageMessagePartCount = messageDimensions.DetermineIntlMessageUnicodePartCount( message.value.length );
    }
    else
    {
        document.getElementById( 'unicodeIndicator' ).style.display = 'none';

        lastSentMessageMessagePartCount = messageDimensions.DetermineIntlMessagePartCount( message.value.length );
    }

    document.getElementById( 'partCount' ).innerHTML = "Parts: " + lastSentMessageMessagePartCount;
}

/******************************************************************************
  UpdateDomesticCounts

    Updates the domestic message counts

******************************************************************************/
function UpdateDomesticCounts( )
{
    var message = document.getElementById( 'messageText' );

    var messageLength = messageDimensions.LimitDomesticMessageLength( message );

    document.getElementById( 'characterCount' ).innerHTML = "Chars: " + messageLength;

    lastSentMessageMessagePartCount = messageDimensions.DetermineDomesticMessagePartCount( messageLength );

    document.getElementById( 'partCount' ).innerHTML = "Parts: " + lastSentMessageMessagePartCount;
    document.getElementById( 'unicodeIndicator' ).style.display = 'none';
}

/******************************************************************************
  FillContactsPopup

    Populates the contacts popup from the address book

******************************************************************************/
function FillContactsPopup( )
{
    var contactPopup = document.getElementById( "contactPopup" );
    contactPopup.options = new Array( );
    contactPopup.options[ 0 ] = new Option( "None Selected", CONTACT_NONE_SELECTED_VALUE );

    if ( SmsPlugin )
    {
        var numContacts = SmsPlugin.numEntries( );
        for ( var i = 0; i < numContacts; i++ )
        {
            contactPopup.options[ i + 1 ] = new Option( SmsPlugin.nameAtIndex( i ), SmsPlugin.mobileNumberAtIndex( i ) );
        }
    }
    else
    {
        alert( "SmsPlugin not loaded." );
    }

    OnContactChanged( );
}


/******************************************************************************
 ******************************************************************************

  Message Sending functions

 ******************************************************************************
******************************************************************************/

/******************************************************************************
  SendHTTPPostRequest

    Sends an HTTP POST request using a primary and failover URL.

    Parameters:
        primaryURL
        failoverURL
        onCompleteCallBack  - function to call on completion
        requestData         - POST data

******************************************************************************/
function SendHTTPPostRequest( primaryURL, failoverURL, onCompleteCallBack, requestData )
{
    try
    {
        var httpClient = new HttpClient( );

        var handleErrorWithCompletion = function( response )
        {
            HandleError( response );
            onCompleteCallBack( );
        };

        var handleSuccessWithCompletion = function( response )
        {
            HandleSuccessResponse( response );
            onCompleteCallBack( );
        };

        httpClient.onError   = handleErrorWithCompletion;
        httpClient.onSuccess = handleSuccessWithCompletion;

        httpClient.sendRequest( HTTP_VERB_POST, primaryURL, failoverURL, HTTP_CONTENT_TYPE_APPLICATION_FORM, requestData );
    }
    catch ( e )
    {
        document.getElementById( 'messageText' ).value = "There was a problem sending the SMS:\n" + e;
    }
}

/******************************************************************************
  HandleSuccessResponse

    Handler for HTTP success responses.

******************************************************************************/
function HandleSuccessResponse( xmlResponse )
{
    try
    {
        // Get the top level <response> element
        var response     = FindChild( xmlResponse, 'response' );
        var responseCode = '';
        var responseText = 'Invalid Server Response';

        if ( response != null )
        {
            var responseCodeElement = FindChild( response, 'responseCode' );
            if ( responseCodeElement )
            {
                responseCode = StripLeadingAndTrailingWhiteSpace( responseCodeElement.firstChild.data );
            }

            var responseTextElement = FindChild( response, 'responseText' );
            if ( responseTextElement )
            {
                responseText = StripLeadingAndTrailingWhiteSpace( responseTextElement.firstChild.data );
            }
        }

        if ( responseCode == OK_RESPONSE )
        {
            // the reponse was a success so clear the message data
            document.getElementById( 'messageText' ).value = "";
        }

        SetStatusMessage( responseText );
    }
    catch ( e )
    {
        SetStatusMessage( "Unable to handle success response:\n" + e );
    }
}

/******************************************************************************
  HandleError

    Handler for HTTP error responses.

******************************************************************************/
function HandleError( errorMessage )
{
    if ( ! errorMessage || ( errorMessage == '' ) )
    {
        errorMessage = "There was a problem sending the SMS:\n" +
                       "Unable to contact the server.  Please check your internet connection and try again in a few minutes.";
    }

    document.getElementById( 'messageText' ).value = errorMessage;

    if ( SmsPlugin )
    {
        SmsPlugin.logMessage( errorMessage );
    }
}

/******************************************************************************
  SendSMS

    Handler for the "Send" button. Sends an SMS request off to the server.

******************************************************************************/
function SendSMS( )
{
    document.getElementById( "doButton" ).src = "Images/Button_down.png";
    document.getElementById( "doButton" ).disabled = true;

    if ( document.getElementById( 'internationalRadio' ).checked )
    {
        SendInternationalMessage( );
    }
    else
    {
        SendDomesticMessage( );
    }

    var contactPopup = document.getElementById( 'contactPopup' );
    widget.setPreferenceForKey( contactPopup.options[ contactPopup.selectedIndex ].value, LAST_CONTACT_KEY );

    document.getElementById( "doButton" ).disabled = false;
}

/******************************************************************************
  ValidateInternationalSend

    Validates the configuration for an international message.

******************************************************************************/
function ValidateInternationalSend( )
{
    var recipientPhoneNumberObj = document.getElementById( "recipientPhoneNumber" );

    if ( ! IsValidInternationalNumber( recipientPhoneNumberObj,
                                       document.getElementById( 'errorMessage' ) ) )
    {
        return false;
    }

    return ValidateMessage( );
}

/******************************************************************************
  SendInternationalMessage

    Sends an international SMS message.

******************************************************************************/
function SendInternationalMessage( )
{
    if ( ! ValidateInternationalSend( ) )
    {
        return;
    }

    SetStatusMessage( "Sending message, please wait..." );

    var recipientPhoneNumber = StripNonDigits( document.getElementById( "recipientPhoneNumber" ).value );
    var postData = "phoneNumber=" + recipientPhoneNumber;

    var message = encodeURIComponent( document.getElementById( 'messageText' ).value );
    postData += "&message=" + message;

    var senderEmail = widget.preferenceForKey( SENDER_EMAIL_KEY );
    postData += "&senderEmail=" + senderEmail;

    var senderPhoneNumber = widget.preferenceForKey( SENDER_MOBILE_KEY );
    if ( senderPhoneNumber )
    {
        postData += "&senderPhoneNumber=" + senderPhoneNumber;

        var senderCarrierId = widget.preferenceForKey( SENDER_CARRIER_ID_KEY );
        if ( ! senderCarrierId )
        {
            senderCarrierId = 0;
        }

        postData += "&senderCarrierId=" + senderCarrierId;
    }

    postData += "&src=widget&version=" + WIDGET_VERSION;

    SendHTTPPostRequest( HTTP_PRIMARY_URL  + "/" + HTTP_CONTEXT_ROOT_INTERNATIONAL,
                         HTTP_FAILOVER_URL + "/" + HTTP_CONTEXT_ROOT_INTERNATIONAL,
                         UpdateInternationalCounts,
                         postData );
}

/******************************************************************************
  ValidateDomesticMessage

    Validates the configuration for a domestic message.

******************************************************************************/
function ValidateDomesticMessage( )
{
    if ( ! SenderInformationExists( ) )
    {
        SetStatusMessage( "Click the 'i' and enter your info!" );
        return false;
    }

    if ( ! IsValidNationalNumber( document.getElementById( "recipientPhoneNumber" ),
                                  document.getElementById( "errorMessage" ),
                                  GetSelectedValueFromPopup( document.getElementById( 'carrierPopup' ) ) ) )
    {
        return false;
    }

    return ValidateMessage( );
}

/******************************************************************************
  SendDomesticMessage

    Sends an domestic SMS message.

******************************************************************************/
function SendDomesticMessage( )
{
    if ( ! ValidateDomesticMessage( ) )
    {
        return;
    }

    SetStatusMessage( "Sending message, please wait..." );

    var recipientPhoneNumber = document.getElementById( "recipientPhoneNumber" ).value;
    var postData = "phoneNumber=" + recipientPhoneNumber;

    var carrierId = GetSelectedValueFromPopup( document.getElementById( 'carrierPopup' ) );
    postData += "&carrierId=" + carrierId;

    var message = document.getElementById( 'messageText' ).value;
    var encodedMessage = encodeURIComponent( message );
    postData += "&message=" + encodedMessage;

    var senderEmail = widget.preferenceForKey( SENDER_EMAIL_KEY );
    if ( senderEmail )
    {
        postData += "&senderEmail=" + senderEmail;
    }

    var senderPhoneNumber = widget.preferenceForKey( SENDER_MOBILE_KEY );
    if ( senderPhoneNumber )
    {
        postData += "&senderPhoneNumber=" + senderPhoneNumber;
    }

    // we need to specify the send-from preference if both email and
    // phone number are sent
    if ( senderEmail && senderPhoneNumber )
    {
        var savedSendFromPreference = widget.preferenceForKey( SENDER_FROM_PREFERENCE_KEY );
        if ( savedSendFromPreference && ( savedSendFromPreference == SEND_FROM_USE_EMAIL_ADDRESS ) )
        {
            postData += "&sendFromPreference=1";
        }
        else
        {
            postData += "&sendFromPreference=0";
        }
    }

    var senderCarrierId = widget.preferenceForKey( SENDER_CARRIER_ID_KEY );
    if ( ! senderCarrierId )
    {
        senderCarrierId = 0;
    }
    postData += "&senderCarrierId=" + senderCarrierId;

    postData += "&src=widget&version=" + WIDGET_VERSION;

    SendHTTPPostRequest( HTTP_PRIMARY_URL  + "/" + HTTP_CONTEXT_ROOT_DOMESTIC,
                         HTTP_FAILOVER_URL + "/" + HTTP_CONTEXT_ROOT_DOMESTIC,
                         UpdateDomesticCounts,
                         postData );

    widget.setPreferenceForKey( GetSelectedValueFromPopup( carrierPopup ), recipientPhoneNumber );

    totalDomesticMessagePartsSent += lastSentMessageMessagePartCount;
    widget.setPreferenceForKey( totalDomesticMessagePartsSent, NUM_SENT_KEY );
}

/******************************************************************************
  ValidateMessage

    Validates the message in preparation for sending

******************************************************************************/
function ValidateMessage( )
{
    var message = document.getElementById( 'messageText' ).value;

    if ( ! message || ( message.length == 0 ) )
    {
        SetStatusMessage( "Please enter a message!" );

        return false;
    }

    return true;
}


/******************************************************************************
 ******************************************************************************

  BACK functionality functions

 ******************************************************************************
******************************************************************************/

/******************************************************************************
  OnSenderCarrierChanged

    Handles sender carrier selection-change events

******************************************************************************/
function OnSenderCarrierChanged( )
{
    var chosenCarrierId = GetSelectedValueFromPopup( document.getElementById( 'senderCarrierPopup' ) );

    if ( chosenCarrierId == INVALID_CARRIER_ID )
    {
        // Don't allow them to select a divider
        document.getElementById( 'senderCarrierPopup' ).options[ 0 ].selected = true;
    }
}

/******************************************************************************
  OnReloadContacts

    Handler for contact reloading button

******************************************************************************/
function OnReloadContacts( )
{
    if ( SmsPlugin )
    {
        SmsPlugin.loadContactsFromAddressBook( );
        FillContactsPopup( );
    }
    else
    {
        alert( "SmsPlugin not loaded." );
    }
}

/******************************************************************************
  OnShowBack

    Handler for transitioning to back pane

******************************************************************************/
function OnShowBack( )
{
    var front = document.getElementById( "front" );
    var back = document.getElementById( "back" );

    document.getElementById( 'backErrorMessage' ).innerHTML = '';

    document.getElementById( 'flipIconRollie' ).style.display = 'none';
    document.getElementById( 'senderInfo' ).style.display = 'block';

    if ( window.widget )
    {
        widget.prepareForTransition( "ToBack" );
    }

    front.style.display = "none";
    back.style.display = "block";

    if ( window.widget )
    {
        setTimeout( 'widget.performTransition( );', 0 );
    }
}

/******************************************************************************
  ValidateSenderPhoneNumber

    Validates the sender phone number

******************************************************************************/
function ValidateSenderPhoneNumber( )
{
    var senderPhoneNumberObj = document.getElementById( "senderPhoneNumber" );

    if ( ( ! senderPhoneNumberObj.value || ( senderPhoneNumberObj.value.length == 0 ) ) )
    {
        // Number is blank, which is ok
        return true;
    }

    var senderCarrierId = GetSelectedValueFromPopup( document.getElementById( 'senderCarrierPopup' ) );

    if ( senderCarrierId == INTERNATIONAL_CARRIER_ID )
    {
        return IsValidInternationalNumber( senderPhoneNumberObj, document.getElementById( "backErrorMessage" ) );
    }

    return IsValidNationalNumber( senderPhoneNumber,
                                  document.getElementById( "backErrorMessage" ),
                                  senderCarrierId );
}

/******************************************************************************
  OnBackPaneDoneButton

    Handler for back pane Done button.

******************************************************************************/
function OnBackPaneDoneButton( )
{
    if ( ! ValidateSenderPhoneNumber( ) )
    {
        return;
    }

    if ( ! IsValidEmailAddress( document.getElementById( 'senderEmail' ).value ) )
    {
        document.getElementById( "backErrorMessage" ).innerHTML = "Please enter a valid email.";
        document.getElementById( 'senderEmail' ).focus( );
        document.getElementById( 'senderEmail' ).select( );

        return;
    }

    document.getElementById( "backErrorMessage" ).innerHTML = "";

    var senderEmail = document.getElementById( 'senderEmail' ).value;
    widget.setPreferenceForKey( senderEmail, SENDER_EMAIL_KEY );

    var senderMobile = document.getElementById( 'senderPhoneNumber' ).value;
    widget.setPreferenceForKey( senderMobile, SENDER_MOBILE_KEY );

    var senderCarrierId = GetSelectedValueFromPopup( document.getElementById( 'senderCarrierPopup' ) );
    widget.setPreferenceForKey( senderCarrierId, SENDER_CARRIER_ID_KEY );

    var sendFromPreference =
        ( document.getElementById( 'sendFromPreferenceCheckBox' ).checked ?
          SEND_FROM_USE_EMAIL_ADDRESS : SEND_FROM_USE_PHONE_NUMBER );
    widget.setPreferenceForKey( sendFromPreference, SENDER_FROM_PREFERENCE_KEY );

    messageDimensions = new MessageDimensions( );

    // Clear the error message on the front, which might have told them to
    // enter their info on the back.
    SetStatusMessage( "" );

    var front = document.getElementById( "front" );
    var back = document.getElementById( "back" );

    if ( window.widget )
    {
        widget.prepareForTransition( "ToFront" );
    }

    back.style.display = "none";
    front.style.display = "block";

    if ( window.widget )
    {
        setTimeout( 'widget.performTransition( );', 0 );
    }
}

/******************************************************************************
  OnCheckForUpdates

    Handler for back pane "Check for updates" button.

******************************************************************************/
function OnCheckForUpdates( )
{
    widget.openURL( CLIENT_VERSION_CHECK_URL + '?src=' + WIDGET_SOURCE +
                                               '&ver=' + WIDGET_VERSION );
}

/******************************************************************************
  OnHelp

    "Help" button.

******************************************************************************/
function OnHelp( )
{
    widget.openURL( HELP_URL );
}


/******************************************************************************
 ******************************************************************************

  Transition icon animation

 ******************************************************************************
******************************************************************************/

var flipShown = false;
var animation =
{
    duration        : 0,
    starttime       : 0,
    to              : 1.0,
    now             : 0.0,
    from            : 0.0,
    firstElement    : null,
    timer           : null
};

function HandleMouseMove( event )
{
    if ( flipShown )
    {
        return;
    }

    if ( animation.timer != null )
    {
        clearInterval( animation.timer );
        animation.timer = null;
    }

    var starttime = ( new Date ).getTime( ) - 13;

    animation.duration = 500;
    animation.starttime = starttime;
    animation.firstElement = document.getElementById( 'flipIcon' );
    animation.timer = setInterval( "animate( );", 13 );
    animation.from = animation.now;
    animation.to = 1.0;

    animate( );

    flipShown = true;
}

function HandleMouseExit( event )
{
    if ( ! flipShown )
    {
        return;
    }

    // fade in the info button
    if ( animation.timer != null )
    {
        clearInterval( animation.timer );
        animation.timer = null;
    }

    var starttime = ( new Date ).getTime( ) - 13;

    animation.duration = 500;
    animation.starttime = starttime;
    animation.firstElement = document.getElementById( 'flipIcon' );
    animation.timer = setInterval( "animate( );", 13 );
    animation.from = animation.now;
    animation.to = 0.0;

    animate( );

    flipShown = false;
}

function animate( )
{
    var T;
    var ease;
    var time = ( new Date ).getTime( );

    T = limit_3( time-animation.starttime, 0, animation.duration );

    if ( T >= animation.duration )
    {
        clearInterval( animation.timer );
        animation.timer = null;
        animation.now = animation.to;
    }
    else
    {
        ease = 0.5 - ( 0.5 * Math.cos( Math.PI * T / animation.duration ) );
        animation.now = computeNextFloat( animation.from, animation.to, ease );
    }

    animation.firstElement.style.opacity = animation.now;
}

function limit_3( a, b, c )
{
    return a < b ? b : ( a > c ? c : a );
}

function computeNextFloat( from, to, ease )
{
    return from + ( to - from ) * ease;
}

function OnIconMouseOver( obj )
{
    var rollieObject = document.getElementById( obj.id + "Rollie" );
    rollieObject.style.display = 'block';
}

function OnIconMouseOut( obj )
{
    var rollieObject = document.getElementById( obj.id + "Rollie" );
    rollieObject.style.display = 'none';
}

var backPaneHelpDescriptions = new Array( );
backPaneHelpDescriptions[ "senderCarrierPopupHelpIcon" ] = "Your mobile phone service provider. Required if a mobile phone number has been entered.";
backPaneHelpDescriptions[ "senderEmailHelpIcon" ]        = "Your email address (required). Can be used as the reply-to address when sending to USA/Canada.";
backPaneHelpDescriptions[ "mobileHelpIcon" ]             = "Your mobile phone number: either 10 digit USA/Canada number, or International (e.g. +44 1234567890). International number cannot be used as the reply-to when sending to USA/Canada.";
function OnHelpIconMouseOver( obj )
{
    OnIconMouseOver( obj );

    var helpMessage = "<table cellpadding='0' cellspacing='0'><tr><td class='helpMessage'>" + backPaneHelpDescriptions[ obj.id ] + '</td></tr></table>';
    document.getElementById( "backErrorMessage" ).innerHTML = helpMessage;
}

function OnHelpIconMouseOut( obj )
{
    OnIconMouseOut( obj );
    document.getElementById( "backErrorMessage" ).innerHTML = '';
}

function SenderInformationExists( )
{
    var senderEmail         = widget.preferenceForKey( SENDER_EMAIL_KEY );
    var senderPhoneNumber   = widget.preferenceForKey( SENDER_MOBILE_KEY );
    var senderCarrierId     = widget.preferenceForKey( SENDER_CARRIER_ID_KEY );

    return ( ( ( senderPhoneNumber  && ( senderPhoneNumber.length >= 10 ) ) &&
               ( senderCarrierId && ( senderCarrierId.length > 0 ) &&
               ( senderCarrierId != 'unknown' ) ) )
                ||
             ( senderEmail     && ( senderEmail.length > 0 ) ) );
}


