/******************************************************************************
    CallWave SMS Widget

    Send free SMS text messages worldwide.

    Copyright (C) 2006 CallWave, Inc.  All Rights Reserved.

******************************************************************************/


/******************************************************************************
  StripLeadingAndTrailingWhiteSpace

    Strips leading and trailing whitespace from a string

******************************************************************************/
function StripLeadingAndTrailingWhiteSpace( str )
{
    var stripped = ( str.replace( /^\s+/, '' ) ).replace( /\s+$/, '' );

    return stripped;
}

/******************************************************************************
  StripNonDigits

    Strips non-digits from a string

******************************************************************************/
function StripNonDigits( str )
{
    var stripped = str.replace( /\D+/, '' );
    return stripped;
}

/******************************************************************************
  IsValidEmailAddress

    Extremely basic email validation

******************************************************************************/
function IsValidEmailAddress( emailAddress )
{
    var MIN_EMAIL_ADDRESS_LENGTH = 5;
    if ( emailAddress != null && emailAddress.match( /^(.+)@(.+)\.(.+)$/ ) && ( emailAddress.length > MIN_EMAIL_ADDRESS_LENGTH ) )
    {
        return true;
    }

    return false;
}


/******************************************************************************
  IsValidInternationalNumber

    Determines if a number represents a valid national number

******************************************************************************/
function IsValidInternationalNumber( phoneNumberObj, errorMessage )
{
    var internationalNumber = StripNonDigits( phoneNumberObj.value );

    // validate the number: >= 8 Characters -> valid.
    if ( ! internationalNumber || ( internationalNumber.length < 8 ) )
    {
        errorMessage.innerHTML = "Please enter a valid international phone number!";
        phoneNumberObj.focus( );
        phoneNumberObj.select( );

        return false;
    }

    return true;
}


/******************************************************************************
  IsValidNationalNumber

    Determines if a number represents a valid national number

******************************************************************************/
function IsValidNationalNumber( phoneNumberObj, errorMessage, carrierId )
{
    var fullNumber = phoneNumberObj.value;

    if ( ! fullNumber || ( fullNumber.length == 0 ) )
    {
        errorMessage.innerHTML = "Please enter a valid phone number!";
        phoneNumberObj.focus( );
        phoneNumberObj.select( );
        return false;
    }

    if ( fullNumber.length != 10 && fullNumber.length != 11 )
    {
        errorMessage.innerHTML = "Please enter a valid area code!";
        phoneNumberObj.focus( );
        phoneNumberObj.select( );
        return false;
    }

    if ( carrierId == INVALID_CARRIER_ID )
    {
        errorMessage.innerHTML = "Please select a carrier!";
        return false;
    }

    return true;
}

/******************************************************************************
  SelectItemInPopup

    Selects an item in a popup menu

******************************************************************************/
function SelectItemInPopup( itemValue, popup )
{
    var i = 0;
    for ( i = 0; i < popup.options.length; ++i )
    {
        if ( popup.options[ i ].value == itemValue )
        {
            popup.selectedIndex = i;

            return;
        }
    }

    popup.selectedIndex = -1;
}

/******************************************************************************
  SelectItemInCustomPopup

    Selects an item in a custom popup menu and then sets the menu text element
    to the selected item

******************************************************************************/
function SelectItemInCustomPopup( itemValue, popup, popupMenuElement )
{
    SelectItemInPopup( itemValue, popup );

    if ( popup.selectedIndex == -1 )
    {
        SelectFirstItemInCustomPopup( popup, popupMenuElement );
        return;
    }

    popupMenuElement.innerText = popup.options[ popup.selectedIndex ].text;
}

/******************************************************************************
  SelectFirstItemInCustomPopup

    Selects the first item in a custom popup menu and then sets the menu text element
    to the selected item

******************************************************************************/
function SelectFirstItemInCustomPopup( popup, popupMenuElement )
{
    popup.options[ 0 ].selected = true;

    popupMenuElement.innerText = popup.options[ popup.selectedIndex ].text;
}

/******************************************************************************
  GetSelectedValueFromPopup

    Retrieves the selected item from a popup menu.

******************************************************************************/
function GetSelectedValueFromPopup( popup )
{
    return popup.options[ popup.selectedIndex ].value;
}

/******************************************************************************
  StringRequiresUnicodeEncoding

    Determines if a string requires unicode encoding.

******************************************************************************/
function StringRequiresUnicodeEncoding( message )
{
    if ( typeof GSM_ALPHABET == "undefined" )
    {
        GSM_ALPHABET = new Array( 64,163,36,165,232,233,249,236,242,216,248,197,229,95,94,123,125,92,91,126,93,124,198,230,223,201,32,33,34,35,164,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,161,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,196,214,209,220,167,191,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,228,246,241,252,224,231,10,12,13 );
    }

    // Workaround Safari bug, check the code of every character.
    var numChars = message.length;

    var unicodeNeeded = false;
    var i = 0;
    for ( i = 0; i < numChars; i++ )
    {
        var currentCharCode = message.charCodeAt( i );

        var charIsGsm = false;
        var j = 0;
        for ( j = 0; j < GSM_ALPHABET.length; j++ )
        {
            if ( currentCharCode == GSM_ALPHABET[j] )
            {
                charIsGsm = true;
                break;
            }
        }

        if ( ! charIsGsm )
        {
            unicodeNeeded = true;
            break;
        }
    }

    return unicodeNeeded;
}

/******************************************************************************
  FindChild

    Scan the children of a given DOM element for a node matching nodeName;
    much more efficient than the standard DOM methods (getElementsByTagName, etc)
    if you know what you're looking for.

******************************************************************************/
function FindChild( element, nodeName )
{
    var child = null;

    for ( child = element.firstChild; child != null; child = child.nextSibling )
    {
        if ( child.nodeName == nodeName )
        {
            return child;
        }
    }

    return null;
}

/******************************************************************************
  PopulateCarrierNames

    Populates a popup menu with a list of carriers.

******************************************************************************/
function PopulateCarrierNames( element, carrierList )
{
    element.options = new Array( );

    var i = 0;
    for ( ; i < carrierList.length; ++i )
    {
        element.options[ element.options.length ] = new Option( carrierList[ i ][ 0 ], carrierList[ i ][ 1 ] );
    }

    element.options[ 0 ].selected = true;
}
